@@ -1,9 +1,9 @@
 from django.contrib import admin
-from django import forms
 from django.utils.translation import ugettext_lazy as _
 from imagekit.admin import AdminThumbnail
 from feincms.admin import editor
 
+from galeria.forms import AlbumAdminForm
 from galeria.models import Album, Picture
 
 
@@ -12,22 +12,34 @@ OPEN_TREE_STR = _('Expand tree')
 COLLAPSE_TREE_STR = _('Collapse tree')
 
 
-class AlbumAdminForm(forms.ModelForm):
+class PictureAdmin(admin.ModelAdmin):
+    list_display = ('thumbnail', 'title', 'is_public', 'album', 'date_added')
+    list_display_links = ('title',)
+    list_editable = ('album', 'is_public')
+    list_filter = ('date_added', 'album', 'is_public')
+    list_per_page = 20
+    prepopulated_fields = {'slug': ('title',)}
+    search_fields = ('title', 'slug', 'description')
 
-    def __init__(self, *args, **kwargs):
-        super(AlbumAdminForm, self).__init__(*args, **kwargs)
-        descendants_set = self.instance.get_descendants(include_self=True).filter(is_public=True)
-        descendants_ids = descendants_set.values('id')
-        picture_set = Picture.objects.public().filter(album__in=descendants_ids).order_by('-date_added')
-        self.fields['cover'].queryset = picture_set
+    thumbnail = AdminThumbnail(
+        image_field='thumbnail_image',
+        template='galeria/admin/thumbnail.html'
+    )
+    thumbnail.short_description = _('thumbnail')
 
-    class Meta:
-        model = Album
+admin.site.register(Picture, PictureAdmin)
 
 
+class InlinePictureAdmin(admin.TabularInline):
+    extra = 1
+    model = Picture
+    prepopulated_fields = {'slug': ('title',)}
+
 class AlbumAdmin(editor.TreeEditor):
     form = AlbumAdminForm
+    inlines = [InlinePictureAdmin]
     list_display = ('title', 'album_cover', 'is_public', 'order')
+    list_editable = ('is_public', 'order')
     list_filter = ['is_public']
     prepopulated_fields = {'slug': ('title',)}
     search_fields = ('title', 'slug', 'description')
@@ -41,20 +53,4 @@ class AlbumAdmin(editor.TreeEditor):
     album_cover.allow_tags = True
     album_cover.short_description = _('cover')
 
-
-class PictureAdmin(admin.ModelAdmin):
-    list_display = ('admin_thumbnail', 'title', 'album', 'date_added', 'is_public')
-    list_display_links = ['title']
-    list_filter = ['date_added', 'album', 'is_public']
-    list_per_page = 20
-    prepopulated_fields = {'slug': ('title',)}
-    search_fields = ('title', 'slug', 'description')
-
-    admin_thumbnail = AdminThumbnail(
-        image_field='thumbnail_image',
-        template='galeria/admin/thumbnail.html'
-    )
-    admin_thumbnail.short_description = _('thumbnail')
-
 admin.site.register(Album, AlbumAdmin)
-admin.site.register(Picture, PictureAdmin)
